// You can import API functions like this from D3.js.
import { 
  select, 
  csv,
  event,
  brushX,
  max,
  range,
  scaleLinear, 
  scaleTime,
  scaleOrdinal,
  schemeCategory10,
  timeParse, 
  timeFormat, 
  extent,
  axisLeft,
  axisBottom,
  line,
  nest
} from 'd3';
const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');
const colorValue = d => d.wl;
const margin = { top: 20, bottom: 110, right: 20, left:50}
const margin2 = { top: 430, bottom: 40, right: 20, left:50}
const innerWidth = width - margin.left - margin.right;
const innerHeight = height - margin.top - margin.bottom;
const innerHeight2 = height - margin2.top - margin2.bottom;
const focus = svg.append('g')
  .attr('class', 'focus')
  .attr('transform', `translate(${margin.left},${margin.top})`);

const context = svg.append("g")
    .attr('class', 'context')
    .attr('transform', `translate(${margin2.left},${margin2.top})`);
const circleRadius = 2;
const xAxisLabel = 'Date';
const yAxisLabel = 'Water level';
const parseDate = timeParse("%d/%m/%Y");
const formatDate = timeFormat("%b %Y");
const yScale = scaleLinear()
 .range([innerHeight, 0]);
const yScale2 = scaleLinear()
 .range([innerHeight2, 0]);
const xScale = scaleTime()
 .range([0,innerWidth]);
const colorScale = scaleOrdinal(schemeCategory10);

let data=[];

svg.append("defs").append("clipPath")
    .attr("id", "clip")
  .append("rect")
    .attr("width", width)
    .attr("height", height);


const render = () => {
  const xValue = d => d.Date;
  const yValue = d => d.value;
    
  const yAxis = axisLeft(yScale);
  const xAxis = axisBottom(xScale);  
   
  const lineGenerator = line()
    .x(d => xScale(xValue(d)))
    .y(d => yScale(yValue(d)));
  
  const lineGenerator2 = line()
    .x(d => xScale(xValue(d)))
    .y(d => yScale2(yValue(d)));
  
  const nested = nest()
    .key(colorValue)
    .entries(data);
 
  colorScale.domain(nested.map(d => d.key));
  
  const yAxisG = focus.append('g')
    .attr('class', 'y-axis');
  yAxisG.call(yAxis);
  
  const xAxisG = focus.append('g')
    .attr('class', 'x-axis')
    .attr('transform', `translate(0,${innerHeight})`);
  xAxisG.call(xAxis);
  
  xAxisG.append('text')
    .attr('class', 'x-axis-label')
    .attr('y', 35)
    .attr('x', innerWidth/2)
    .attr('fill', 'black')
    .text(xAxisLabel)
  
  const xAxisGC = context.append('g')
    .attr('class', 'x-axis')
    .attr('transform', `translate(0,${innerHeight2})`);
  xAxisGC.call(xAxis);
  
  xAxisGC.append('text')
    .attr('class', 'x-axis-label')
    .attr('y', 35)
    .attr('x', innerWidth/2)
    .attr('fill', 'black')
    .text(xAxisLabel)

  yAxisG.append('text')
    .attr("transform", "rotate(-90)")
    .attr('class', 'y-axis-label')
    .attr('y', -30)
    .attr('x', -innerHeight/2)
    .attr('fill', 'black')
    .attr('text-anchor','middle')
    .text(yAxisLabel)
  
  focus.selectAll('.line-path').data(nested)
    .enter().append('path')
    .attr('class', 'line-path')
    .attr('d', d => lineGenerator(d.values))
    .attr('stroke', d => colorScale(d.key));
  
  focus.selectAll('.circle-focus').data(data)
    .attr("clip-path", "url(#clip)")
  .enter().append('circle')
    .attr('class','circle-focus')
    .attr('cx', d => xScale(xValue(d)))
    .attr('cy', d => yScale(yValue(d)))
    .attr('r', circleRadius);
////////////////////////////////////////////////
  
  context.selectAll('.circle').data(data)
    .attr("clip-path", "url(#clip)")
  .enter().append('circle')
    .attr('class','circle')
    .attr('cx', d => xScale(xValue(d)))
    .attr('cy', d => yScale2(yValue(d)))
    .attr('r', circleRadius);
  
  context.selectAll('.line-path').data(nested)
    .enter().append('path')
      .attr('class', 'line-path')
      .attr('d', d => lineGenerator2(d.values))
      .attr('stroke', d => colorScale(d.key));
  
  let brushed = () => {
  let selection = event.selection || xScale.range();
  xScale.domain(selection.map(xScale.invert, xScale));

  focus.selectAll(".line-path")
    .attr("d", d => lineGenerator(d.values));
  focus.selectAll("circle")
    .attr('cx', d => xScale(xValue(d)))
    .attr('cy', d => yScale(yValue(d)));
  focus.select(".x-axis").call(xAxis);
}
 
  const brush = brushX()
                .extent([[0,0],[innerWidth,innerHeight2]])
                .on("brush end",brushed);
  
  select(".context")
      .call(brush)
      .call(brush.move, xScale.range())
}




const wls =["OW2DWL", "OW2DSWL"];
csv('https://vizhub.com/careyshan/datasets/monthlyViz.csv')
  .then(loadedData => {
  loadedData.forEach(row => {
    wls.forEach(wl => {
    if(row[wl]!==undefined){
    data.push({
			Date:parseDate(row.Date),
      wl,
      
      value: +row[wl]
    	})
    }})
  })
  xScale.domain(extent(data, d => d.Date));
  yScale.domain(extent(data, d => d.value));
  yScale2.domain(extent(data, d => d.value));
});

setTimeout(()=>{
	render();
}, 2000)